
require 'chef/resource'
require 'chef/provider'

if defined?(ChefSpec)
  def start_container(resource_name)
    ChefSpec::Matchers::ResourceMatcher.new('container', :start, resource_name)
  end

  def stop_container(resource_name)
    ChefSpec::Matchers::ResourceMatcher.new('container', :stop, resource_name)
  end

  def create_container(resource_name)
    ChefSpec::Matchers::ResourceMatcher.new('container', :create, resource_name)
  end

  def destroy_container(resource_name)
    ChefSpec::Matchers::ResourceMatcher.new(
      'container',
      :destroy,
      resource_name
    )
  end
end

class Chef
  class Resource
    # Chef resource for an individual container
    class Container < Chef::Resource
      identity_attr :container_name

      def initialize(name, run_context = nil)
        super
        @resource_name = :container
        @container_name = name
        @options = {}
        @provider = Chef::Provider::Container
        @action = :create
        @allowed_actions += [:start, :stop, :destroy, :create]
      end

      def container_name(arg = nil)
        set_or_return(:container_name, arg, kind_of: [String])
      end

      def options(arg = nil)
        set_or_return(:options, arg, kind_of: [Hash])
      end
    end
  end

  class Provider
    # Chef provider for an individual container
    class Container < Chef::Provider
      attr_reader :ct

      def initialize(new_resource, run_context)
        super(new_resource, run_context)
      end

      def whyrun_supported?
        true
      end

      def load_current_resource
        require 'lxc'
        @ct = LXC::Container.new(new_resource.container_name)
      end

      def action_create
        return if ct.defined?
        converge_by("create container '#{ct.name}'") do
          opts = @new_resource.options
          template = opts[:template] || 'ubuntu'
          block_device = opts[:block_device] || nil
          template_options = opts[:template_options] || []
          flags = opts[:flags] || 0
          ct.create(template, block_device, flags, template_options)
        end
      end

      def action_stop
        return unless ct.running?
        converge_by("stop container '#{ct.name}'") do
          ct.stop
        end
      end

      def action_start
        return if ct.running?
        converge_by("start container '#{ct.name}'") do
          ct.start
        end
      end

      def action_destroy
        return unless ct.defined?
        converge_by("destroy container '#{ct.name}'") do
          ct.destroy
        end
      end
    end
  end
end
